#!/usr/bin/perl -T
##
use strict;
use Getopt::Long;

$ENV{'PATH'}='/sbin:/usr/bin:/usr/sbin';  # secure enough?

my $wigner_pattern ='^188\.185\.';


my $opt_verbose=0;
my $opt_noaction=0;
my $opt_help =0;

sub get_my_own_ip() {
    my $ipaddr ='';
    my $iproute=`busybox iproute list`;
    if ($iproute =~ /default\s+via.*dev (\w+)/s) {
	my $defdev = $1;
	print "#INFO: default network device: $defdev\n" if ($opt_verbose);
	my $ipaddr_show=`busybox ipaddr show dev '$defdev'`;
	if($ipaddr_show =~ /\s+inet\s+\S+\s+brd\s+(\d+\.\d+\.\d+\.\d+)\s+scope/s) {
	    $ipaddr = $1;
	    print "#INFO: my IP: $ipaddr\n" if ($opt_verbose);
	} else {
	    print "#WARN: default network device ' $defdev' has no IPv4 address\n";
	}
    } else {
	print "#WARN: no default network device found\n";
    }
    return $ipaddr;
}


sub get_fileservers( $ ) {
    my @fileservers_raw=`vos listaddrs -noresolve -noauth`;
    my @fileservers;
    for my $s (@fileservers_raw) {
	if ($s =~ /^(\d+\.\d+\.\d+\.\d+)$/) {
	    push(@fileservers, $1);
	}
    }
    return @fileservers;
}

sub get_mycell() {
    my $mycell = '';
    my $mycell_raw=`fs wscell`;
    if($mycell_raw =~ m/belongs to cell .([a-z.]+)./s) {
	$mycell = $1;
    }
    print "#INFO: this AFS client is in cell $mycell\n" if $opt_verbose;
    return $mycell;
}

sub get_dbservers( $ ) {
    my $mycell=shift;
    my $cellservdb='NoCellServDBFoundInStandardPlaces';
    for my $f ('/usr/vice/etc/CellServDB', '/etc/openafs/CellServDB') {
	if ( -r $f) {
	    $cellservdb = $f;
	    last;
	}
    }
    if (! open(CELLSERVDB, "<$cellservdb")) {
	print "#ERROR: could not open $cellservdb:$!";
	return;
    }
    my @allcells = <CELLSERVDB>; # small enough for 'slurp'
    close(CELLSERVDB);
    my $foundcell = 0;
    my @dbservers;
    for my $line (@allcells) {
	if ($line =~ m/^>$mycell\s+/) {   # start of 'our' block
	    $foundcell =1;
	    next;
	} elsif ($foundcell and $line =~ m/^\[?(\d+\.\d+\.\d+\.\d+)\]?\s+/) {
	    push(@dbservers, $1);
	    next;
	} elsif ($foundcell and $line =~ m/^>/) { # next block
	    last;
	}
    }
    return @dbservers;
}

sub usage( $ ) {
    my $ret = shift;
    print <<EOFusage;
Usage: $0 [-n|--noaction] [-v|--verbose]

Tells the AFS client which AFS servers in the local cell are closer
than the others, based on its own IPv4 address and hardcoded networks.
Unless "--noaction" is specified, this needs to be run as superuser.

EOFusage
exit($ret);
}
# main
GetOptions ("help" => \$opt_help,
	    "noaction" => \$opt_noaction,
	    "verbose" => \$opt_verbose) or usage(1);
if($opt_help) {
    usage(0);
}

if($> != 0 and not $opt_noaction) {
    print "#ERROR: needs to be run as root\n";
    exit(1);
}

my $cell = get_mycell();

my $at_wigner = ( get_my_own_ip() =~ m/$wigner_pattern/ );  # default:no.
if ($at_wigner) {
   print "#INFO: will prefer servers matching $wigner_pattern\n" if $opt_verbose;
} else {
   print "#INFO: will avoid  servers matching $wigner_pattern\n" if $opt_verbose;
}

my $pref_fs_close=25000; # default is 30000 for same net, 40000 for other
my $pref_fs_remote=35000;
my @fsprefs;
for my $ip (get_fileservers($cell)) {
    if (($ip =~  m/$wigner_pattern/) == $at_wigner) {
	# either both client+server at Wigner, or not => prefer
	push(@fsprefs,"$ip $pref_fs_close");
    } else {
	push(@fsprefs,"$ip $pref_fs_remote");
    }
}

my $pref_db_close=8000;  # default is 10000
my $pref_db_remote=12000;
my @dbprefs;
for my $ip (get_dbservers($cell)) {
    if (($ip =~  m/$wigner_pattern/) == $at_wigner) {
	# either both client+server at Wigner, or not => prefer
	push(@dbprefs,"$ip $pref_db_close");
    } else {
	push(@dbprefs,"$ip $pref_db_remote");
    }
}

if($#fsprefs > 0) {
    my $cmd = "fs setserverprefs -servers ".join(" ", @fsprefs);
     if($opt_noaction) {
	print "# would run $cmd\n";
    } else {
	print "#INFO: running $cmd\n" if $opt_verbose;
	system($cmd);
    }
}
if($#dbprefs > 0) {
    my $cmd = "fs setserverprefs -vlservers ".join(" ", @dbprefs);
    if($opt_noaction) {
	print "# would run $cmd\n";
    } else {
	print "#INFO: running $cmd\n" if $opt_verbose;
	system($cmd);
    }
}
